@@ -3,7 +3,6 @@ find_nodes <- function(tokens, tquery, block=NULL, use_index=TRUE, name=NA, fill
   tokens = as_tokenindex(tokens)  
   block = get_long_ids(block)
   
-  #nodes_list = list()
   nodes = filter_tokens(tokens, lookup=tquery$lookup, .G_ID=tquery$g_id, .BLOCK=block, use_index=use_index)
   if (nrow(nodes) == 0) return(NULL)
   nodes = subset(nodes, select = c('doc_id','sentence','token_id'))
@@ -15,6 +14,7 @@ find_nodes <- function(tokens, tquery, block=NULL, use_index=TRUE, name=NA, fill
     data.table::setnames(nodes, old = 'token_id', new='.ID')
     if (!is.na(tquery$label)) nodes[,(tquery$label) := .ID]
   } 
+
   if (is.null(nodes)) return(NULL)  
   if (nrow(nodes) == 0) return(NULL)
 
@@ -23,11 +23,10 @@ find_nodes <- function(tokens, tquery, block=NULL, use_index=TRUE, name=NA, fill
   nodes = get_unique_patterns(nodes)
   if (!root_dist) nodes$.ROOT_DIST = NULL
   
-  #print(nodes)
   if (fill) nodes = add_fill(tokens, nodes, tquery, block=nodes)
-  #print(nodes)
+
   
-  nodes = create_unique_key(nodes, name)
+  nodes = create_unique_key(nodes, name, tquery)
   if (melt) {
     nodes = melt_nodes_list(nodes)
   }
@@ -36,9 +35,9 @@ find_nodes <- function(tokens, tquery, block=NULL, use_index=TRUE, name=NA, fill
 
 find_nested <- function(tokens, nodes, tquery, block, fill, block_loop) {
   .ID = NULL; .MATCH_ID = NULL
+
   nodes = rec_find(tokens, ids=nodes, ql=tquery$nested, block=block, fill=fill, block_loop=block_loop)
   
-  
   if (nrow(nodes) == 0) return(NULL)
   nodes[, .ID := .MATCH_ID]
   data.table::setcolorder(nodes, c('.ID', setdiff(colnames(nodes), '.ID')))
@@ -64,7 +63,6 @@ add_fill <- function(tokens, nodes, tquery, block, level=1) {
     }
   } 
   
-
   if (any(is_fill)) {
     if (is.na(tquery$label)) {
       if (level == 1) match_id = '.ID' else return(nodes)
@@ -90,21 +88,26 @@ add_fill <- function(tokens, nodes, tquery, block, level=1) {
   unique(nodes)
 }
 
-create_unique_key <- function(nodes, name){
-  #if (ncol(nodes) > 3) {
-  #  key = paste0(name, '(', nodes$.ID, ':', do.call(paste, args = c(nodes[,-(1:3)], sep='.')), ')')
-  #} else {
-  #  key = paste0(name, '(', nodes$.ID, ')')
-  #}      
-  id_col = setdiff(colnames(nodes), c('doc_id','sentence','.ID'))[1]
-  #key = paste0(name, '#', nodes$doc_id, '.', nodes$sentence, '.', match(nodes$.ID, unique(nodes$.ID)))
+get_top_label <- function(tquery) {
+  ## get the first label in a tquery.
+  if (!is.na(tquery$label)) return(tquery$label)
+  if (!is.null(tquery$nested)) {
+    for (nested in tquery$nested) {
+      label = get_top_label(nested)
+      if (!is.na(label)) return(label)
+    }
+  }
+  return(NA)
+}
+
+create_unique_key <- function(nodes, name, tquery){
+  id_col = get_top_label(tquery)
   if (!is.na(name)) {
     key = paste0(name, '#', nodes$doc_id, '.', nodes$sentence, '.', nodes[[id_col]])
   } else {
     key = paste0(nodes$doc_id, '.', nodes$sentence, '.', nodes[[id_col]])
   }
 
-
   nodes$.ID = paste0(nodes$doc_id, '...', nodes$sentence, '...', nodes$.ID) ## quick fix for matching on 3 columns
   key = key[match(nodes$.ID, nodes$.ID)] ## give same id to nodes with same .ID
   #key = paste0(name, '#', 1:nrow(nodes))
@@ -114,7 +117,6 @@ create_unique_key <- function(nodes, name){
 
 get_root_dist <- function(tokens, nodes) {
   .ROOT_DIST = NULL
-  
   tf = token_family(tokens, unique(data.table(doc_id=nodes$doc_id, sentence=nodes$sentence, token_id=nodes$.ID)), 
                     depth=Inf, level='parents', minimal=TRUE, show_level=TRUE, replace=TRUE)
   tf = data.table::setorderv(tf, cols = '.FILL_LEVEL', order = -1)
